#!/usr/bin/env python3
# _*_ coding:utf-8 _*_
# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.
# SPDX-License-Identifier: GPL-2.0-only

import functools
import inspect
import multiprocessing
import os
import subprocess
import sys
import time
from os.path import dirname
from socketserver import ThreadingMixIn
from xmlrpc.client import ServerProxy
from xmlrpc.server import SimpleXMLRPCServer

sys.path.append(dirname(dirname(dirname(os.path.abspath(__file__)))))

from setting import conf
# from src.depends.colorama import Fore

client_project_path = "/".join(conf.ROOT_DIR.split("/")[3:])
exclude_files = [
    "apps",
    "report",
    "public",
    "__pycache__",
    "node_modules",
    ".pytest_cache",
    ".vscode",
    ".idea",
    ".git",
    "site",
    "docs",
    "site",
    "README.md",
    "README.en.md",
    "RELEASE.md",
]


def _feeling_good(cmd):
    subprocess.Popen(cmd, shell=True)


def _base_env_check():
    sudo = f"echo '{conf.PASSWORD}' | sudo -S"
    os.system("rm -rf ~/.ssh/known_hosts")
    if "StrictHostKeyChecking no" not in os.popen("cat /etc/ssh/ssh_config").read():
        os.system(
            f'{sudo} sed -i "s/#   StrictHostKeyChecking ask/ StrictHostKeyChecking no/g" /etc/ssh/ssh_config'
        )
    if "/home/" not in conf.ROOT_DIR:
        raise EnvironmentError


def _exclude():
    exclude_str = ""
    for i in exclude_files:
        exclude_str += f"--exclude='{i}' "

    return exclude_str


def _ssh(ip, password, user):
    return f"sshpass -p '{password}' ssh {user}@{ip}"


def _transfer_appname(ip, password, user, transfer_appname):
    os.system(
        f"sshpass -p '{password}' rsync -av -e ssh --exclude='__pycache__' "
        f"{conf.APPS_PATH}/{transfer_appname} {user}@{ip}:~/{client_project_path}/apps/"
    )


def _transfer_to_client(ip, password, user):
    os.system(f'''{_ssh(ip, password, user)} "mkdir -p ~/{client_project_path}" > /dev/null 2>&1''')
    os.system(
        f"sshpass -p '{password}' rsync -av -e ssh {_exclude()} {conf.ROOT_DIR}/* "
        f"{user}@{ip}:~/{client_project_path}/"
    )
    os.system(
        f'''{_ssh(ip, password, user)} "cd ~/{client_project_path}/ && mkdir apps && touch apps/__init__.py" > /dev/null 2>&1'''
    )
    os.system(
        f'''{_ssh(ip, password, user)} "cd ~/{client_project_path}/apps/ && touch REMOTE" > /dev/null 2>&1'''
    )
    if (
            not os.popen(
                f'''{_ssh(ip, password, user)} "cd ~/{client_project_path}/ && ls env_ok"'''
            ).read().strip()
    ):
        os.system(
            f'{_ssh(ip, password, user)} "cd ~/{client_project_path}/ && bash env.sh -p {password} && touch env_ok"'
        )


def _start_client_service(ip, password, user, filename):
    if not os.path.exists(f"{conf.REPORT_PATH}/logs/"):
        os.makedirs(f"{conf.REPORT_PATH}/logs/")
    _cmd = (
        f"nohup {_ssh(ip, password, user)} "
        f'"cd ~/{client_project_path}/src/remotectl/ && '
        f'pipenv run python {filename}" >>  {conf.REPORT_PATH}/logs/{filename}.log 2>&1 &'
    )
    # print(Fore.GREEN, _cmd, Fore.RESET)
    p = multiprocessing.Process(target=_feeling_good, args=(_cmd,))
    p.start()
    time.sleep(1)


def _restart_client_service(ip, password, user, filename):
    GREP_LIST = (
        "grep",
    )
    cmd = ""
    for i in GREP_LIST:
        cmd += f"grep -v {i} | "
    os.system(
        f"{_ssh(ip, password, user)} "
        f'''"ps -ef | grep {filename} | {cmd}cut -c 9-15 | xargs kill -9 > /dev/null 2>&1"'''
    )
    time.sleep(1)
    _start_client_service(ip, password, user, filename)


def check_rpc_started(filename):
    def deco(func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            user = kwargs.get("user") or args[0]
            ip = kwargs.get("ip") or args[1]
            if not user or not ip:
                raise ValueError("user and ip are required")
            password = kwargs.get("password") or (args[2] if len(args) >= 3 else conf.PASSWORD)
            transfer_appname = kwargs.get("transfer_appname")
            restart_service = kwargs.get("restart_service")

            _base_env_check()
            if transfer_appname:
                _transfer_appname(ip, password, user, transfer_appname)
            tool_status = os.popen(
                f'''{_ssh(ip, password, user)} "ps -aux |  grep {filename} | grep -v grep"'''
            ).read()
            if not tool_status:
                _transfer_to_client(ip, password, user)
                _start_client_service(ip, password, user, filename)
            if restart_service:
                _restart_client_service(ip, password, user, filename)
            res = func(*args, **kwargs)

            return res

        return wrapper

    return deco


def remote_client(ip, port):
    try:
        import zerorpc
    except ImportError:
        raise ImportError("Please install zerorpc")

    c = zerorpc.Client(timeout=20, heartbeat=None)
    try:
        c.connect(f"tcp://{ip}:{port}")
        return c
    except Exception as e:
        raise e


def remote_server(obj, port):
    try:
        import zerorpc
    except ImportError:
        raise ImportError("Please install zerorpc")
    server = zerorpc.Server(obj)
    server.bind(f"tcp://0.0.0.0:{port}")
    server.run()


class ThreadXMLRPCServer(ThreadingMixIn, SimpleXMLRPCServer): ...


def _remote_client(ip, port):
    client = ServerProxy(f"http://{ip}:{port}", allow_none=True)
    return client


def _remote_server(obj, port):
    server = ThreadXMLRPCServer(("0.0.0.0", port), allow_none=True)
    for func, _ in inspect.getmembers(obj):
        if not func.startswith("_"):
            server.register_function(getattr(obj, func), func)
    server.serve_forever()


if __name__ == '__main__':
    from src import Src

    _remote_server(Src(), 4242)



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_
# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.
# SPDX-License-Identifier: GPL-2.0-only
import tomli


def read_toml(file_path):
    with open(file_path, "rb") as config_file:
        toml_dict = tomli.load(config_file)
    return toml_dict


if __name__ == "__main__":
    from setting import conf

    read_toml(f"{conf.ROOT_DIR}/playbook.toml")



def git_control(parser=None, sub_parser_git=None):
    sub_parser_git.add_argument(
        "-a",
        "--app",
        default="",
        help="应用名称：apps/autotest_deepin_music 或 autotest_deepin_music",
    )
    sub_parser_git.add_argument("-u", "--git_user", default="", help="git仓库用户名")
    sub_parser_git.add_argument("-p", "--git_password", default="", help="git仓库地密码")
    sub_parser_git.add_argument("-l", "--git_url", default="", help="git仓库地址")
    sub_parser_git.add_argument("-b", "--branch_or_tag", default="", help="分支或Tag")
    sub_parser_git.add_argument("-d", "--depth", default="", help="git仓库克隆深度")
    sub_parser_git.add_argument("-s", "--startdate", default="", help="统计开始时间")
    sub_parser_git.add_argument("-e", "--enddate", default="", help="统计结束时间")
    args = parser.parse_args()
    from src.rtk._base import Args
    from setting import conf

    git_kwargs = {
        Args.app_name.value: args.app or conf.APP_NAME,
        Args.git_url.value: args.git_url or conf.GIT_URL,
        Args.git_user.value: args.git_user or conf.GIT_USER,
        Args.git_password.value: args.git_password or conf.GIT_PASSWORD,
        Args.branch.value: args.branch_or_tag or conf.BRANCH,
        Args.depth.value: args.depth or conf.DEPTH,
        Args.startdate.value: args.startdate or conf.START_DATE,
        Args.enddate.value: args.enddate or conf.END_DATE,
    }
    from src.git.check import check_git_installed

    if git_kwargs.get(Args.git_url.value):
        if all(
            [
                git_kwargs.get(Args.git_user.value),
                git_kwargs.get(Args.git_password.value),
            ]
        ):
            from src.git.clone import sslclone as git_clone
        else:
            from src.git.clone import clone as git_clone
        check_git_installed()
        git_clone(**git_kwargs)
    if git_kwargs.get(Args.app_name.value):
        if git_kwargs.get(Args.startdate.value):
            from src.git.code_statistics import CodeStatistics

            check_git_installed()
            CodeStatistics(**git_kwargs).codex()
        else:
            print("-s/--startdate 参数未传入")
    else:
        print("-a/--app 参数未传入")



"""
    Python cFonts
    =============
    Sexy fonts for the console.

    :license: GNU GPLv2
    :author: Frost Ming<mianghong@gmail.com>
"""
import argparse
import sys
from typing import List, Optional

from . import consts
from .__version__ import __version__
from .core import render, say


class CFontsArgumentParser(argparse.ArgumentParser):
    def format_help(self) -> str:
        formatter = self._get_formatter()

        # description
        formatter._add_item(
            lambda x: x,
            [render("cfonts", gradient=["red", "green"], space=False) + "\n"],
        )
        formatter.add_text(self.description)

        # usage
        formatter.add_usage(
            self.usage or "", self._actions, self._mutually_exclusive_groups
        )

        # positionals, optionals and user-defined groups
        for action_group in self._action_groups:
            formatter.start_section(action_group.title)
            formatter.add_text(action_group.description)
            formatter.add_arguments(action_group._group_actions)
            formatter.end_section()

        # epilog
        formatter.add_text(self.epilog)

        # determine help from format above
        return formatter.format_help()


def parse_args() -> argparse.Namespace:
    parser = CFontsArgumentParser(
        "cfonts",
        description="This is a tool for sexy fonts in the console. "
        "Give your cli some love.",
    )
    parser.add_argument(
        "-V",
        "--version",
        action="version",
        version="{} {}".format(
            render("cfonts", font="console", colors=["candy"], space=False), __version__
        ),
    )

    parser.add_argument(
        "-f",
        "--font",
        default=consts.FontFaces.block.value,
        choices=[f.value for f in consts.FontFaces],
        help="Use to define the font face",
    )
    parser.add_argument(
        "-c",
        "--colors",
        default=consts.Colors.system.value,
        help="Use to define the font color",
    )
    parser.add_argument(
        "-b",
        "--background",
        default=consts.BgColors.transparent.value,
        help="Use to define the background color",
    )
    parser.add_argument(
        "-a",
        "--align",
        default="left",
        choices=consts.ALIGNMENT,
        help="Use to align the text output",
    )
    parser.add_argument(
        "-l", "--letter-spacing", type=int, help="Use to define the letter spacing"
    )
    parser.add_argument(
        "-z", "--line-height", default=1, help="Use to define the line height"
    )
    parser.add_argument(
        "-s",
        "--spaceless",
        dest="space",
        default=True,
        action="store_false",
        help="Use to define the background color",
    )

    parser.add_argument(
        "-m",
        "--max-length",
        default=0,
        help="Use to define the amount of maximum characters per line",
    )
    parser.add_argument(
        "-g",
        "--gradient",
        help="Define gradient colors(separated by comma)",
    )
    parser.add_argument(
        "-i",
        "--independent-gradient",
        action="store_true",
        help="Set this option to re-calculate the gradient colors for each new line."
        "Only works in combination with the gradient option.",
    )
    parser.add_argument(
        "-t",
        "--transition-gradient",
        dest="transition",
        action="store_true",
        help="Set this option to generate your own gradients. Each color set "
        "in the gradient option will then be transitioned to directly.",
    )
    parser.add_argument(
        "-r",
        "--raw",
        action="store_true",
        help="Instead of printing the colors, print the ANSI escape codes, so it "
        "can be copied and pasted into a Python print statement.",
    )
    parser.add_argument("text")
    if len(sys.argv) == 1:
        parser.print_help()
        sys.exit(1)

    args = parser.parse_args()
    return args


def main() -> None:
    args = parse_args()

    colors = [c.strip() for c in args.colors.split(",")]
    if args.gradient:
        gradient: Optional[List[str]] = [g.strip() for g in args.gradient.split(",")]
    else:
        gradient = None
    options = {
        "font": args.font,
        "colors": colors,
        "background": args.background,
        "align": args.align,
        "line_height": args.line_height,
        "space": args.space,
        "max_length": args.max_length,
        "gradient": gradient,
        "independent_gradient": args.independent_gradient,
        "transition": args.transition,
        "raw": args.raw,
    }
    if args.letter_spacing is not None:
        options["letter_spacing"] = args.letter_spacing
    say(args.text, **options)



from funnylog import logger

from setting.globalconfig import GlobalConfig


def local_runner(parser, sub_parser_run, cmd_args=None):
    """本地执行"""
    sub_parser_run.add_argument(
        "-a",
        "--app",
        default="",
        help="应用名称：apps/autotest_deepin_music 或 autotest_deepin_music",
    )
    sub_parser_run.add_argument(
        "-k", "--keywords", default="", help="用例的关键词,支持and/or/not逻辑组合"
    )
    sub_parser_run.add_argument(
        "-t", "--tags", default="", help="用例的标签,支持and/or/not逻辑组合"
    )
    sub_parser_run.add_argument("--rerun", default="", help="失败重跑次数")
    sub_parser_run.add_argument(
        "--record_failed_case", default="", help="失败录屏从第几次失败开始录制视频"
    )
    sub_parser_run.add_argument("--clean", choices=["yes", ""], default="", help="清理环境")
    sub_parser_run.add_argument("--report_formats", default="", help="测试报告格式")
    sub_parser_run.add_argument("--max_fail", default="", help="最大失败率")
    sub_parser_run.add_argument("--log_level", default="", help="日志输出级别")
    sub_parser_run.add_argument("--timeout", default="", help="单条用例超时时间")
    sub_parser_run.add_argument("--resolution", default="", help="检查分辨率")
    sub_parser_run.add_argument("--debug", default="", help="调试模式")
    sub_parser_run.add_argument(
        "--noskip",
        choices=["yes", ""],
        default="",
        help="csv文件里面标记了skip跳过的用例不生效",
    )
    sub_parser_run.add_argument(
        "--ifixed", choices=["yes", ""], default="", help="fixed不生效，仅通过skip跳过用例"
    )
    sub_parser_run.add_argument(
        "--send_pms", choices=["", "async", "finish"], default="", help="数据回填"
    )
    sub_parser_run.add_argument("--task_id", default="", help="测试单ID")
    sub_parser_run.add_argument(
        "--trigger", choices=["", "auto", "hand"], default="", help="触发者"
    )
    sub_parser_run.add_argument("-f", "--case_file", default="", help="根据文件执行用例")
    sub_parser_run.add_argument("--deb_path", default="", help="需要安装deb包的本地路径")
    sub_parser_run.add_argument("-u", "--pms_user", default="", help="pms 用户名")
    sub_parser_run.add_argument("-p", "--pms_password", default="", help="pms 密码")
    sub_parser_run.add_argument("--suite_id", default="", help="pms 测试套ID")
    sub_parser_run.add_argument("--pms_info_file", default="", help="pms 信息文件")
    sub_parser_run.add_argument("--top", default="", help="过程中记录top命令中的值")
    sub_parser_run.add_argument(
        "--lastfailed",
        action="store_const",
        const=True,
        default=False,
        help="仅执行上次失败用例",
    )
    sub_parser_run.add_argument(
        "--duringfail",
        action="store_const",
        const=True,
        default=False,
        help="测试过程中立即显示报错",
    )
    sub_parser_run.add_argument("--repeat", default="", help="指定用例执行次数")
    sub_parser_run.add_argument("--project_name", default="", help="工程名称（写入json文件）")
    sub_parser_run.add_argument("--build_location", default="", help="构建地区（写入json文件）")
    sub_parser_run.add_argument("--line", default="", help="执行的业务线（写入json文件）")
    sub_parser_run.add_argument("--autostart", default="", help="重启类场景开启letmego执行方案")
    sub_parser_run.add_argument("--slaves", default="", help="远程测试机")
    args = parser.parse_args()
    from src.rtk._base import Args

    local_kwargs = {
        Args.app_name.value: args.app,
        Args.keywords.value: args.keywords,
        Args.tags.value: args.tags,
        Args.reruns.value: args.rerun,
        Args.record_failed_case.value: args.record_failed_case,
        Args.clean.value: args.clean,
        Args.report_formats.value: args.report_formats,
        Args.max_fail.value: args.max_fail,
        Args.log_level.value: args.log_level,
        Args.timeout.value: args.timeout,
        Args.debug.value: args.debug,
        Args.noskip.value: args.noskip,
        Args.ifixed.value: args.ifixed,
        Args.send_pms.value: args.send_pms,
        Args.task_id.value: args.task_id,
        Args.trigger.value: args.trigger,
        Args.resolution.value: args.resolution,
        Args.case_file.value: args.case_file,
        Args.deb_path.value: args.deb_path,
        Args.pms_user.value: args.pms_user,
        Args.pms_password.value: args.pms_password,
        Args.suite_id.value: args.suite_id,
        Args.pms_info_file.value: args.pms_info_file,
        Args.top.value: args.top,
        Args.lastfailed.value: args.lastfailed,
        Args.duringfail.value: args.duringfail,
        Args.repeat.value: args.repeat,
        Args.project_name.value: args.project_name,
        Args.build_location.value: args.build_location,
        Args.line.value: args.line,
        Args.autostart.value: args.autostart,
        Args.slaves.value: args.slaves,
    }
    if local_kwargs.get(Args.autostart.value) or GlobalConfig.AUTOSTART:
        try:
            import letmego

            letmego.conf.setting.PASSWORD = GlobalConfig.PASSWORD
            letmego.register_autostart_service(
                user=GlobalConfig.USERNAME,
                working_directory=GlobalConfig.ROOT_DIR,
                cmd=f"pipenv run python manage.py {' '.join(cmd_args)}",
            )
        except ModuleNotFoundError:
            ...
    return local_kwargs, args


def remote_runner(parser, sub_parser_remote):
    sub_parser_remote.add_argument(
        "-c",
        "--clients",
        default="",
        help=(
            "远程机器的user@ip:password,多个机器用'/'连接,"
            "如果password不传入,默认取setting/remote.ini中CLIENT_PASSWORD的值,"
            "比如: uos@10.8.13.xx:1 或 uos@10.8.13.xx"
        ),
    )
    sub_parser_remote.add_argument(
        "-s",
        "--send_code",
        action="store_const",
        const=True,
        default=False,
        help="发送代码到测试机（不含report目录）",
    )
    sub_parser_remote.add_argument(
        "-e",
        "--build_env",
        action="store_const",
        const=True,
        default=False,
        help="搭建测试环境,如果为yes，不管send_code是否为yes都会发送代码到测试机.",
    )
    sub_parser_remote.add_argument(
        "-cp", "--client_password", default="", help="测试机密码（全局）"
    )
    sub_parser_remote.add_argument(
        "--git_url", default="", help="git仓库地址"
    )
    sub_parser_remote.add_argument(
        "--git_user", default="", help="git仓库用户名"
    )
    sub_parser_remote.add_argument(
        "--git_password", default="", help="git仓库密码"
    )
    sub_parser_remote.add_argument(
        "-b", "--branch_or_tag", default="", help="分支或Tag"
    )
    sub_parser_remote.add_argument(
        "-d", "--depth", default="", help="git仓库克隆深度"
    )
    sub_parser_remote.add_argument(
        "-y",
        "--parallel",
        default="",
        help=(
            "yes:表示所有测试机并行跑，执行相同的测试用例;"
            "no:表示测试机分布式执行，服务端会根据收集到的测试用例自动分配给各个测试机执行。"
        ),
    )
    sub_parser_remote.add_argument(
        "--json_backfill_base_url", default="", help="json报告回填的接口地址"
    )
    sub_parser_remote.add_argument(
        "--json_backfill_task_id", default="", help="json报告回填所属任务id"
    )
    sub_parser_remote.add_argument(
        "--json_backfill_user", default="", help="json报告回填的用户名"
    )
    sub_parser_remote.add_argument(
        "--json_backfill_password", default="", help="json报告回填的密码"
    )
    sub_parser_remote.add_argument(
        "--json_backfill_custom_api", default="", help="json报告回填的自定义api，默认是/api/youqu/yqresult/"
    )

    local_kwargs, args = local_runner(parser, sub_parser_remote)
    from src.rtk._base import Args

    remote_kwargs = {
        Args.clients.value: args.clients,
        Args.send_code.value: args.send_code,
        Args.build_env.value: args.build_env,
        Args.client_password.value: args.client_password,
        Args.git_url.value: args.git_url or GlobalConfig.GIT_URL,
        Args.git_user.value: args.git_user or GlobalConfig.GIT_USER,
        Args.git_password.value: args.git_password or GlobalConfig.GIT_PASSWORD,
        Args.branch.value: args.branch_or_tag or GlobalConfig.BRANCH,
        Args.depth.value: args.depth or GlobalConfig.DEPTH,
        Args.parallel.value: args.parallel,
        Args.json_backfill_base_url.value: args.json_backfill_base_url,
        Args.json_backfill_task_id.value: args.json_backfill_task_id,
        Args.json_backfill_user.value: args.json_backfill_user,
        Args.json_backfill_password.value: args.json_backfill_password,
        Args.json_backfill_custom_api.value: args.json_backfill_custom_api,
    }
    _remote_kwargs = {
        "remote_kwargs": remote_kwargs,
        "local_kwargs": local_kwargs,
    }

    if remote_kwargs.get(Args.git_url.value):
        if all(
                [
                    remote_kwargs.get(Args.git_user.value),
                    remote_kwargs.get(Args.git_password.value),
                ]
        ):
            from src.git.clone import sslclone as git_clone
        else:
            from src.git.clone import clone as git_clone
        from src.git.check import check_git_installed
        check_git_installed()
        git_clone(**remote_kwargs)
    return _remote_kwargs


def csv_control(parser=None, sub_parser_csv=None):
    sub_parser_csv.add_argument(
        "-a",
        "--app",
        default="",
        help="应用名称：apps/autotest_deepin_music 或 autotest_deepin_music",
    )
    sub_parser_csv.add_argument("-k", "--keywords", default="", help="用例的关键词")
    sub_parser_csv.add_argument("-t", "--tags", default="", help="用例的标签")
    sub_parser_csv.add_argument(
        "-p2c",
        "--pyid2csv",
        action="store_const",
        const=True,
        default=False,
        help="将用例py文件的case id同步到对应的csv文件中",
    )
    sub_parser_csv.add_argument(
        "-ec", "--export_csv_file", default="", help="导出csv文件名称，比如：case_list.csv"
    )
    args = parser.parse_args()
    from src.rtk._base import Args

    csv_kwargs = {
        Args.app_name.value: args.app,
        Args.keywords.value: args.keywords,
        Args.tags.value: args.tags,
        Args.pyid2csv.value: args.pyid2csv,
        Args.export_csv_file.value: args.export_csv_file,
        "collection_only": True,
    }
    if csv_kwargs.get(Args.pyid2csv.value) or GlobalConfig.PY_ID_TO_CSV:
        from src.csvctl import CsvControl

        _csv = CsvControl(csv_kwargs.get(Args.app_name.value))
        _csv.delete_mark_in_csv_if_not_exists_py()
        _csv.async_mark_to_csv()
    elif csv_kwargs.get(Args.export_csv_file.value):
        from src.rtk.local_runner import LocalRunner

        LocalRunner(**csv_kwargs).local_run()
    else:
        logger.error(
            f"需要传递一些有用参数或配置项：{Args.pyid2csv.value} 或 {Args.export_csv_file.value}"
            "，您可以使用 -h 或 --help 查看支持的参数"
        )



# -*- coding: utf-8 -*-
# This file was modified by UnionTech Software Technology Co., Ltd. in 2022/08/11
# pylint: disable=C0114,W0105,C0103,R0205,C0116,R1720,C0415，W0707,C0209,R0202,C0115,R0903,C0301
from __future__ import absolute_import, division, print_function, unicode_literals
import locale
import os
import sys
import pwd

"""
The configuration module.
"""
__author__ = "Zack Cerza <zcerza@redhat.com>, David Malcolm <dmalcolm@redhat.com>"


def _userTmpDir(baseName):
    # i.e. /tmp/dogtail-foo
    logname = os.getenv('LOGNAME', default=pwd.getpwuid(os.getuid())[0])
    return '-'.join(('/'.join(('/tmp', baseName)), logname))


class _Config(object):

    """
    Contains configuration parameters for the dogtail run.

    scratchDir(str):
    Directory where things like screenshots are stored.

    dataDir(str):
    Directory where related data files are located.

    logDir(str):
    Directory where dogtail.tc.TC*-generated logs are stored.

    scriptName(str) [Read-Only]:
    The name of the script being run.

    encoding(str)
    The encoding for text, used by dogtail.tc.TCString .

    actionDelay(float):
    The delay after an action is executed.

    typingDelay(float):
    The delay after a character is typed on the keyboard.

    runInterval(float):
    The interval at which dogtail.utils.run() and dogtail.procedural.run()
    check to see if the application has started up.

    runTimeout(int):
    The timeout after which dogtail.utils.run() and dogtail.procedural.run()
    give up on looking for the newly-started application.

    searchBackoffDuration (float):
    Time in seconds for which to delay when a search fails.

    searchWarningThreshold (int):
    Number of retries before logging the individual attempts at a search.

    searchCutoffCount (int):
    Number of times to retry when a search fails.

    searchShowingOnly (boolean):
    Whether to only search among nodes that are currently being shown.

    defaultDelay (float):
    Default time in seconds to sleep when delaying.

    childrenLimit (int):
    When there are a very large number of children of a node, only return
    this many, starting with the first.

    debugSearching (boolean):
    Whether to write info on search backoff and retry to the debug log.

    debugSleep (boolean):
    Whether to log whenever we sleep to the debug log.

    debugSearchPaths (boolean):
    Whether we should write out debug info when running the SearchPath
    routines.

    absoluteNodePaths (boolean):
    Whether we should identify nodes in the logs with long 'abcolute paths', or
    merely with a short 'relative path'. FIXME: give examples

    ensureSensitivity (boolean):
    Should we check that ui nodes are sensitive (not 'greyed out') before
    performing actions on them? If this is True (the default) it will raise
    an exception if this happens. Can set to False as a workaround for apps
    and toolkits that don't report sensitivity properly.

    debugTranslation (boolean):
    Whether we should write out debug information from the translation/i18n
    subsystem.

    blinkOnActions (boolean):
    Whether we should blink a rectangle around a Node when an action is
    performed on it.

    fatalErrors (boolean):
    Whether errors encountered in dogtail.procedural should be considered
    fatal. If True, exceptions will be raised. If False, warnings will be
    passed to the debug logger.

    checkForA11y (boolean):
    Whether to check if accessibility is enabled. If not, just assume it is
    (default True).

    logDebugToFile (boolean):
    Whether to write debug output to a log file.

    logDebugToStdOut (boolean):
    Whether to print log output to console or not (default True).

    reversed (boolean):
    When traversing nodes, is reverse iteration adopted?(default False).

    """
    @property
    def scriptName(self):
        return os.path.basename(sys.argv[0]).replace('.py', '')

    @property
    def encoding(self):
        return locale.getpreferredencoding().lower()

    defaults = {
        # Storage
        'scratchDir': '/'.join((_userTmpDir('dogtail'), '')),
        'dataDir': '/'.join((_userTmpDir('dogtail'), 'data', '')),
        'logDir': '/'.join((_userTmpDir('dogtail'), 'logs', '')),
        'scriptName': scriptName.fget(None),
        'encoding': encoding.fget(None),
        'configFile': None,
        'baseFile': None,

        # Timing and Limits
        'actionDelay': 1.0,
        'typingDelay': 0.1,
        'runInterval': 0.5,
        'runTimeout': 30,
        'searchBackoffDuration': 0.5,
        'searchWarningThreshold': 3,
        'searchCutoffCount': 20,
        'searchShowingOnly': False,
        'defaultDelay': 0.5,
        'childrenLimit': 100,

        # Debug
        'debugSearching': False,
        'debugSleep': False,
        'debugSearchPaths': False,
        'logDebugToStdOut': True,
        'absoluteNodePaths': False,
        'ensureSensitivity': False,
        'debugTranslation': False,
        'blinkOnActions': False,
        'fatalErrors': False,
        'checkForA11y': True,

        # Logging
        'logDebugToFile': True,

        # reversed
        'reversed': True
    }

    options = {}

    invalidValue = "__INVALID__"

    def __init__(self):
        _Config.__createDir(_Config.defaults['scratchDir'])
        _Config.__createDir(_Config.defaults['logDir'])
        _Config.__createDir(_Config.defaults['dataDir'])

    def __setattr__(self, name, value):
        if name not in config.defaults:
            raise AttributeError(name + " is not a valid option.")

        elif _Config.defaults[name] != value or \
                _Config.options.get(name, _Config.invalidValue) != value:
            if 'Dir' in name:
                _Config.__createDir(value)
                if value[-1] != os.path.sep:
                    value = value + os.path.sep
            elif name == 'logDebugToFile':
                from . import logging
                logging.debugLogger = logging.Logger('debug', value)
            _Config.options[name] = value

    def __getattr__(self, name):
        try:
            return _Config.options[name]
        except KeyError:
            try:
                return _Config.defaults[name]
            except KeyError:
                raise AttributeError("%s is not a valid option." % name)

    def __createDir(cls, dirName, perms=0o777):
        """
        Creates a directory (if it doesn't currently exist), creating any
        parent directories it needs.

        If perms is None, create with python's default permissions.
        """
        dirName = os.path.abspath(dirName)
        # print "Checking for %s ..." % dirName,
        if not os.path.isdir(dirName):
            if perms:
                umask = os.umask(0)
                os.makedirs(dirName, perms)
                os.umask(umask)
            else:
                # This is probably a dead code - no other functions call this without the
                # permissions set
                os.makedirs(dirName)  # pragma: no cover
    __createDir = classmethod(__createDir)

    def load(self, dictt):
        """
        Loads values from dict, preserving any options already set that are not overridden.
        """
        _Config.options.update(dictt)

    def reset(self):
        """
        Resets all settings to their defaults.
        """
        _Config.options = {}

        # 显示服务器

    DISPLAY_SERVER = (
                         os.popen("cat ~/.xsession-errors | grep XDG_SESSION_TYPE | head -n 1")
                             .read()
                             .split("=")[-1]
                             .strip("\n")
                     ) or ("x11" if os.popen("ps -ef | grep -v grep | grep kwin_x11").read() else "wayland")

    class DisplayServer:
        wayland = "wayland"
        x11 = "x11"

    IS_X11 = (DISPLAY_SERVER == DisplayServer.x11)
    IS_WAYLAND = (DISPLAY_SERVER == DisplayServer.wayland)


config = _Config()



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals
from .config import config
from .i18n import TranslatableString
from .logging import debugLogger as logger
from gi.repository import GLib
import re
from time import sleep

"""
Predicates that can be used when searching for nodes.
"""
__author__ = 'David Malcolm <dmalcolm@redhat.com>'


def stringMatches(scriptName, reportedName):
    assert isinstance(scriptName, TranslatableString)

    return scriptName.matchedBy(reportedName)


def makeScriptRecursiveArgument(isRecursive, defaultValue):
    if isRecursive == defaultValue:
        return ""
    else:
        return ", recursive=%s" % isRecursive


def makeCamel(string):
    """
    Convert string to camelCaps
    """
    string = str(string)
    # FIXME: this function is probably really fragile, lots of difficult cases
    # here

    # Sanitize string, replacing bad characters with spaces:
    for char in ":;!@#$%^&*()-+=_~`\\/?|[]{}<>,.\t\n\r\"'":
        string = string.replace(char, " ")
    words = string.strip().split(" ")
    for word in words:
        word.strip
    result = ""
    firstWord = True
    for word in words:
        lowercaseWord = word.lower()
        if firstWord:
            result += lowercaseWord
            firstWord = False
        else:
            result += lowercaseWord.capitalize()
    return result


class Predicate(object):
    """
    Abstract base class representing a predicate function on nodes.

    It's more than just a function in that it has data and can describe itself
    """

    def satisfiedByNode(self, node):
        """
        Pure virtual method returning a boolean if the predicate is satisfied by the node
        """
        raise NotImplementedError

    def describeSearchResult(self, node):
        raise NotImplementedError

    def makeScriptMethodCall(self, isRecursive):
        """
        Method to generate a string containing a (hopefully) readable search
        method call on a node (to be used when generating Python source code in
        the event recorder)
        """
        raise NotImplementedError

    def makeScriptVariableName(self):
        """
        Method to generate a string containing a (hopefully) readable name
        for a Node instance variable that would be the result of a search on
        this predicate (to be used when generating Python source code in the
        event recorder).
        """
        raise NotImplementedError

    def __eq__(self, other):
        """
        Predicates are considered equal if they are of the same subclass and
        have the same data
        """
        # print "predeq: self:%s"%self
        # print "               other:%s"%other
        # print "predeq: selfdict:%s"%self.__dict__
        # print "               otherdict:%s"%other.__dict__

        if not isinstance(self, type(other)):
            return False
        else:
            return self.__dict__ == other.__dict__


class IsAnApplicationNamed(Predicate):
    """Search subclass that looks for an application by name"""

    def __init__(self, appName):
        self.appName = TranslatableString(appName)
        self.debugName = self.describeSearchResult()
        self.satisfiedByNode = self._genCompareFunc()

    def _genCompareFunc(self):
        def satisfiedByNode(node):
            try:
                return node.roleName == 'application' and stringMatches(self.appName, node.name)
            except GLib.GError as e:
                if re.match(r"name :[0-9]+\.[0-9]+ was not provided", e.message):
                    logger.log("Dogtail: warning: omiting possibly broken at-spi application record")
                    return False
                else:
                    try:
                        sleep(config.defaults['searchWarningThreshold'])
                        return node.roleName == 'application' and stringMatches(self.appName, node.name)
                    except GLib.GError:
                        logger.log("Dogtail: warning: application may be hanging")
                        return False
        return satisfiedByNode

    def describeSearchResult(self):
        return '%s application' % self.appName

    def makeScriptMethodCall(self, isRecursive):
        # ignores the isRecursive parameter
        return "application(%s)" % self.appName

    def makeScriptVariableName(self):
        return makeCamel(self.appName) + "App"


class GenericPredicate(Predicate):
    """
    SubtreePredicate subclass that takes various optional search fields
    """

    def __init__(self, name=None, roleName=None, description=None, label=None, debugName=None):
        if name:
            self.name = TranslatableString(name)
        else:
            self.name = None
        self.roleName = roleName
        self.description = description
        if label:
            self.label = TranslatableString(label)
        else:
            self.label = None

        if debugName:
            self.debugName = debugName
        else:
            if label:
                self.debugName = "labelled '%s'" % self.label
            else:
                self.debugName = "child with"
            if name:
                self.debugName = str(self.debugName) + str(" name=%s") % str(self.name)
            if roleName:
                self.debugName = str(self.debugName) + str(" roleName='%s'") % str(roleName)
            if description:
                self.debugName = str(self.debugName) + str(" description='%s'") % str(description)
        assert self.debugName

        self.satisfiedByNode = self._genCompareFunc()

    def _genCompareFunc(self):
        def satisfiedByNode(node):
            # labelled nodes are handled specially:
            if self.label:
                # this reverses the search; we're looking for a node with LABELLED_BY
                # and then checking the label, rather than looking for a label and
                # then returning whatever LABEL_FOR targets
                if node.labeller:
                    return stringMatches(self.label, node.labeller.name)
                else:
                    return False
            else:
                # Ensure the node matches any criteria that were set:
                try:
                    if self.name:
                        if not stringMatches(self.name, node.name):
                            return False
                    if self.roleName:
                        if self.roleName != node.roleName:
                            return False
                    if self.description:
                        if self.description != node.description:
                            return False
                except GLib.GError as e:
                    if re.match(r"name :[0-9]+\.[0-9]+ was not provided", e.message):
                        logger.log("Dogtail: warning: omiting possibly broken at-spi application record")
                        return False
                    else:
                        raise e
                return True
        return satisfiedByNode

    def describeSearchResult(self):
        return self.debugName

    def makeScriptMethodCall(self, isRecursive):
        if self.label:
            args = "label=%s" % self.label
        else:
            args = ""
            if self.name:
                print(self.name)
                args += " name=%s" % self.name
            if self.roleName:
                args += " roleName='%s'" % self.roleName
            if self.description:
                args += " description='%s'" % self.description
        return "child(%s%s)" % (args, makeScriptRecursiveArgument(isRecursive, True))

    def makeScriptVariableName(self):
        if self.label:
            return makeCamel(self.label) + "Node"
        else:
            if self.name:
                return makeCamel(self.name) + "Node"
            if self.roleName:
                return makeCamel(self.roleName) + "Node"
            if self.description:
                return makeCamel(self.description) + "Node"


class IsNamed(Predicate):
    """Predicate subclass that looks simply by name"""

    def __init__(self, name):
        self.name = TranslatableString(name)
        self.debugName = self.describeSearchResult()
        self.satisfiedByNode = self._genCompareFunc()

    def _genCompareFunc(self):
        def satisfiedByNode(node):
            return stringMatches(self.name, node.name)
        return satisfiedByNode

    def describeSearchResult(self):
        return "named %s" % self.name

    def makeScriptMethodCall(self, isRecursive):
        return "child(name=%s%s)" % (self.name, makeScriptRecursiveArgument(isRecursive, True))

    def makeScriptVariableName(self):
        return makeCamel(self.name) + "Node"


class IsAWindowNamed(Predicate):
    """
    Predicate subclass that looks for a top-level window by name
    """

    def __init__(self, windowName):
        self.windowName = TranslatableString(windowName)
        self.debugName = self.describeSearchResult()
        self.satisfiedByNode = self._genCompareFunc()

    def _genCompareFunc(self):
        def satisfiedByNode(node):
            return node.roleName == 'frame' and stringMatches(self.windowName, node.name)
        return satisfiedByNode

    def describeSearchResult(self):
        return "%s window" % self.windowName

    def makeScriptMethodCall(self, isRecursive):
        return "window(%s%s)" % (self.windowName, makeScriptRecursiveArgument(isRecursive, False))

    def makeScriptVariableName(self):
        return makeCamel(self.windowName) + "Win"


class IsAWindow(Predicate):
    """
    Predicate subclass that looks for top-level windows
    """

    def __init__(self):
        self.satisfiedByNode = lambda node: node.roleName == 'frame'

    def describeSearchResult(self):
        return "window"


class IsADialogNamed(Predicate):
    """
    Predicate subclass that looks for a top-level dialog by name
    """

    def __init__(self, dialogName):
        self.dialogName = TranslatableString(dialogName)
        self.debugName = self.describeSearchResult()
        self.satisfiedByNode = self._genCompareFunc()

    def _genCompareFunc(self):
        def satisfiedByNode(node):
            return node.roleName == 'dialog' and stringMatches(self.dialogName, node.name)
        return satisfiedByNode

    def describeSearchResult(self):
        return '%s dialog' % self.dialogName

    def makeScriptMethodCall(self, isRecursive):
        return "dialog(%s%s)" % (self.dialogName, makeScriptRecursiveArgument(isRecursive, False))

    def makeScriptVariableName(self):
        return makeCamel(self.dialogName) + "Dlg"


class IsLabelledBy(Predicate):
    """
    Predicate: is this node labelled by another node
    """
    pass


class IsLabelledAs(Predicate):
    """
    Predicate: is this node labelled with the text string (i.e. by another node with that as a name)
    """

    def __init__(self, labelText):
        self.labelText = TranslatableString(labelText)
        self.debugName = self.describeSearchResult()
        self.satisfiedByNode = self._genCompareFunc()

    def _genCompareFunc(self):
        def satisfiedByNode(node):
            # FIXME
            if node.labeller:
                return stringMatches(self.labelText, node.labeller.name)
            else:
                return False
        return satisfiedByNode

    def describeSearchResult(self):
        return 'labelled %s' % self.labelText

    def makeScriptMethodCall(self, isRecursive):
        return "child(label=%s%s)" % (self.labelText, makeScriptRecursiveArgument(isRecursive, True))

    def makeScriptVariableName(self):
        return makeCamel(self.labelText) + "Node"


class IsAMenuNamed(Predicate):
    """
    Predicate subclass that looks for a menu by name
    """

    def __init__(self, menuName):
        self.menuName = TranslatableString(menuName)
        self.debugName = self.describeSearchResult()
        self.satisfiedByNode = lambda node: node.roleName == 'menu' and stringMatches(self.menuName, node.name)

    def describeSearchResult(self):
        return '%s menu' % (self.menuName)

    def makeScriptMethodCall(self, isRecursive):
        return "menu(%s%s)" % (self.menuName, makeScriptRecursiveArgument(isRecursive, True))

    def makeScriptVariableName(self):
        return makeCamel(self.menuName) + "Menu"


class IsAMenuItemNamed(Predicate):
    """
    Predicate subclass that looks for a menu item by name
    """

    def __init__(self, menuItemName):
        self.menuItemName = TranslatableString(menuItemName)
        self.debugName = self.describeSearchResult()
        self.satisfiedByNode = lambda node: \
            node.roleName.endswith('menu item') and stringMatches(self.menuItemName, node.name)

    def describeSearchResult(self):
        return '%s menuitem' % (self.menuItemName)

    def makeScriptMethodCall(self, isRecursive):
        return "menuItem(%s%s)" % (self.menuItemName, makeScriptRecursiveArgument(isRecursive, True))

    def makeScriptVariableName(self):
        return makeCamel(self.menuItemName) + "MenuItem"


class IsATextEntryNamed(Predicate):
    """
    Predicate subclass that looks for a text entry by name
    """

    def __init__(self, textEntryName):
        self.textEntryName = TranslatableString(textEntryName)
        self.debugName = self.describeSearchResult()
        self.satisfiedByNode = lambda node: node.roleName == 'text' and stringMatches(self.textEntryName, node.name)

    def describeSearchResult(self):
        return '%s textentry' % (self.textEntryName)

    def makeScriptMethodCall(self, isRecursive):
        return "textentry(%s%s)" % (self.textEntryName, makeScriptRecursiveArgument(isRecursive, True))

    def makeScriptVariableName(self):
        return makeCamel(self.textEntryName) + "Entry"


class IsAButtonNamed(Predicate):
    """
    Predicate subclass that looks for a button by name
    """

    def __init__(self, buttonName):
        self.buttonName = TranslatableString(buttonName)
        self.debugName = self.describeSearchResult()
        self.satisfiedByNode = lambda node: node.roleName == 'push button' and stringMatches(self.buttonName, node.name)

    def describeSearchResult(self):
        return '%s button' % (self.buttonName)

    def makeScriptMethodCall(self, isRecursive):
        return "button(%s%s)" % (self.buttonName, makeScriptRecursiveArgument(isRecursive, True))

    def makeScriptVariableName(self):
        return makeCamel(self.buttonName) + "Button"


class IsATabNamed(Predicate):
    """
    Predicate subclass that looks for a tab by name
    """

    def __init__(self, tabName):
        self.tabName = TranslatableString(tabName)
        self.debugName = self.describeSearchResult()
        self.satisfiedByNode = lambda node: node.roleName == 'page tab' and stringMatches(self.tabName, node.name)

    def describeSearchResult(self):
        return '%s tab' % (self.tabName)

    def makeScriptMethodCall(self, isRecursive):
        return "tab(%s%s)" % (self.tabName, makeScriptRecursiveArgument(isRecursive, True))

    def makeScriptVariableName(self):
        return makeCamel(self.tabName) + "Tab"



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_

# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.

# SPDX-License-Identifier: GPL-2.0-only
# pylint: disable=C0114
import math
from xmlrpc.client import ServerProxy

from setting.globalconfig import GlobalConfig

try:
    import numpy as np

    GET_NP_FROM_RPC = False
except ModuleNotFoundError:
    GET_NP_FROM_RPC = True

from src import logger


class Calculate:
    """Calculate"""

    @staticmethod
    def coordinate_distance(start: tuple, end: tuple):
        """
         计算两个坐标之间的直线距离
        :param start: 起始坐标
        :param end: 终止坐标
        :return: 两点之间距离
        """
        if GET_NP_FROM_RPC:
            server = ServerProxy(GlobalConfig.OPENCV_SERVER_HOST, allow_none=True)
            return server.coordinate_distance(start, end)
        position_start = np.array(start)
        position_end = np.array(end)
        position = position_end - position_start
        return math.hypot(position[0], position[1])

    @staticmethod
    def translational_coordinates(start: tuple, relative: tuple):
        """
         计算坐标平移
        :param start: 起始坐标
        :param relative: 平移的相对坐标
        :return: 平移后的坐标
        """
        if GET_NP_FROM_RPC:
            server = ServerProxy(GlobalConfig.OPENCV_SERVER_HOST, allow_none=True)
            return server.translational_coordinates(start, relative)
        position_start = np.array(start)
        position_end = np.array(relative)
        _c = np.sum([position_start, position_end], axis=0)
        logger.debug(f"平移坐标 “{_c}”")
        return _c



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_

# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.

# SPDX-License-Identifier: GPL-2.0-only
# pylint: disable=C0114
from time import sleep as slp
from setting.globalconfig import GlobalConfig
from src import logger


def sleep(second: [float, int]):
    """
     重写sleep方法
     1、增加等待时间的日志
     2、根据不同CPU架构进行放大
    :param second: 等待时间
    :return:
    """
    sys_arch = GlobalConfig.SYS_ARCH
    multiple = float(GlobalConfig.slp_cfg.get(sys_arch))
    mult_sec = second * multiple
    logger.debug(f"sleep {second} s [{sys_arch} * {multiple} = {mult_sec}]")
    slp(mult_sec)



from setting.globalconfig import GlobalConfig


def pms_control(parser=None, sub_parser_pms=None):
    """pms相关功能命令行参数"""
    sub_parser_pms.add_argument(
        "-a",
        "--app",
        default="",
        help="应用名称：apps/autotest_deepin_music 或 autotest_deepin_music",
    )
    sub_parser_pms.add_argument("-u", "--pms_user", default="", help="pms 用户名")
    sub_parser_pms.add_argument("-p", "--pms_password", default="", help="pms 密码")
    sub_parser_pms.add_argument(
        "-plc",
        "--pms_link_csv",
        default="",
        help="pms 和 csv 的映射关系，比如：music:81/album:82，多个配置使用'/'分隔",
    )
    sub_parser_pms.add_argument(
        "-p2c",
        "--pms2csv",
        action="store_const",
        const=True,
        default=False,
        help="从PMS爬取用例标签到csv文件",
    )
    sub_parser_pms.add_argument(
        "--send2task", choices=["yes", ""], default="", help="回填数据到pms测试单"
    )
    sub_parser_pms.add_argument("--task_id", default="", help="测试单ID")
    sub_parser_pms.add_argument(
        "--trigger", choices=["auto", "hand", ""], default="", help="触发者"
    )
    args = parser.parse_args()
    from src.rtk._base import Args

    pms_kwargs = {
        Args.app_name.value: args.app,
        Args.pms_user.value: args.pms_user,
        Args.pms_password.value: args.pms_password,
        Args.pms2csv.value: args.pms2csv,
        Args.pms_link_csv.value: args.pms_link_csv,
        Args.send2task.value: args.send2task,
        Args.task_id.value: args.task_id or GlobalConfig.TASK_ID,
        Args.trigger.value: args.trigger or GlobalConfig.TRIGGER,
    }
    if pms_kwargs.get(Args.pms2csv.value):
        from src.pms.pms2csv import Pms2Csv

        Pms2Csv(
            app_name=pms_kwargs.get(Args.app_name.value),
            user=pms_kwargs.get(Args.pms_user.value) or GlobalConfig.PMS_USER,
            password=pms_kwargs.get(Args.pms_password.value) or GlobalConfig.PMS_PASSWORD,
            pms_link_csv=pms_kwargs.get(Args.pms_link_csv.value),
        ).write_new_csv()
    elif (
        pms_kwargs.get(Args.send2task.value)
        and pms_kwargs.get(Args.task_id.value)
        and pms_kwargs.get(Args.trigger.value) == "hand"
    ):
        from src.pms.send2pms import Send2Pms

        Send2Pms().send2pms(
            Send2Pms.case_res_path(pms_kwargs.get(Args.task_id.value)),
            Send2Pms.data_send_result_csv(pms_kwargs.get(Args.task_id.value)),
        )
    else:
        raise ValueError



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_

# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.

# SPDX-License-Identifier: GPL-2.0-only
# pylint: disable=C0114
# pylint: disable=wrong-import-position
import os
import sys
import traceback
from argparse import ArgumentParser

from setting.globalconfig import GlobalConfig
from setting.globalconfig import SystemPath

os.environ["DISPLAY"] = ":0"
os.environ["PIPENV_VERBOSITY"] = "-1"
os.environ["XAUTHORITY"] = f"{GlobalConfig.HOME}/.Xauthority"

for i in SystemPath:
    if i.value in sys.path:
        continue
    sys.path.append(i.value)

from funnylog import logger
from funnylog.conf import setting as log_setting

log_setting.LOG_FILE_PATH = GlobalConfig.REPORT_PATH


class Manage:
    __author__ = "mikigo<huangmingqiang@uniontech.com>"

    def __init__(self):
        from src.plugins.mng import trim
        from src.plugins.mng import help_tip
        from src.plugins.mng import SubCmd

        trim()
        logger(GlobalConfig.LOG_LEVEL)

        self.cmd_args = sys.argv[1:]
        if not self.cmd_args:
            print(help_tip())
            sys.exit(1)
        parser = ArgumentParser(epilog=self.__author__)
        subparsers = parser.add_subparsers(help="子命令")

        sub_parser_remote = subparsers.add_parser(SubCmd.remote.value)
        sub_parser_run = subparsers.add_parser(SubCmd.run.value)
        sub_parser_pms = subparsers.add_parser(SubCmd.pmsctl.value)
        sub_parser_csv = subparsers.add_parser(SubCmd.csvctl.value)
        sub_parser_git = subparsers.add_parser(SubCmd.git.value)

        if self.cmd_args[0] == SubCmd.remote.value:
            from src.rtk._cargo import remote_runner
            from src.rtk.remote_runner import RemoteRunner

            remote_kwargs = remote_runner(parser, sub_parser_remote)
            RemoteRunner(**remote_kwargs).remote_run()

        elif self.cmd_args[0] == SubCmd.run.value:
            from src.rtk._cargo import local_runner
            from src.rtk.local_runner import LocalRunner

            _local_kwargs, _ = local_runner(parser, sub_parser_run, self.cmd_args)
            LocalRunner(**_local_kwargs).local_run()

        elif self.cmd_args[0] == SubCmd.pmsctl.value:
            from src.pms._cargo import pms_control

            pms_control(parser, sub_parser_pms)

        elif self.cmd_args[0] == SubCmd.csvctl.value:
            from src.rtk._cargo import csv_control

            csv_control(parser, sub_parser_csv)

        elif self.cmd_args[0] == SubCmd.startapp.value:
            start_config_log = f"{SubCmd.startapp.value} 后面直接加工程名称"
            try:
                if self.cmd_args[1] in ("-h", "--help"):
                    print(start_config_log)
                    sys.exit(0)
                from src.plugins.mng import start_app

                start_app(self.cmd_args[1])
            except IndexError:
                logger.error(f"参数异常: {start_config_log}")

        elif self.cmd_args[0] == SubCmd.git.value:
            from src.git._cargo import git_control

            git_control(parser, sub_parser_git)

        elif self.cmd_args[0] in ["-h", "--help"]:
            print(help_tip())

        else:
            print(f"参数异常 \033[0;31m{self.cmd_args}\033[0m!\n{help_tip()}")


if __name__ == "__main__":
    try:
        Manage()
    except Exception as exc:
        traceback.print_exception(*sys.exc_info())



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals
import Accessibility

"""
Superclasses for application wrappers

Subclass these classes if you want to create application wrappers, e.g.:
http://svn.gnome.org/viewvc/dogtail-tests/trunk/appwrappers/dogtail/appwrappers/gedit.py?view=markup
"""
__author__ = "Zack Cerza <zcerza@redhat.com>"


def makeWrapperClass(wrappedClass, name):  # pragma: no cover
    class klass(object):

        def __init__(self, obj):
            self.obj = obj

        def __getattr__(self, name):
            if name == 'obj':
                return self.__dict__['obj']
            return getattr(self.obj, name)

        def __setattr__(self, name, value):
            if name == 'obj':
                self.__dict__['obj'] = value
            else:
                return setattr(self.obj, name, value)

    klass.__name__ = name
    return klass

Application = makeWrapperClass(Accessibility.Application, "WrappedApplication")
Node = makeWrapperClass(Accessibility.Accessible, "WrappedNode")



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_

# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.

# SPDX-License-Identifier: GPL-2.0-only
# pylint: disable=C0114
# pylint: disable=R0913,C0413,E0401
import sys

from funnylog import log as log
from funnylog import logger as logger
from funnylog.conf import setting as log_setting
from setting.globalconfig import GlobalConfig

log_setting.LOG_FILE_PATH = GlobalConfig.REPORT_PATH

from setting.globalconfig import SystemPath

for i in SystemPath:
    if i.value not in sys.path:
        sys.path.append(i.value)

from src.dbus_utils import DbusUtils as DbusUtils
from src.assert_common import AssertCommon as AssertCommon
from src.calculate import Calculate as Calculate
from src.cmdctl import CmdCtl as CmdCtl
from src.dogtail_utils import DogtailUtils as DogtailUtils
from src.image_utils import ImageUtils as ImageUtils
from src.ocr_utils import OCRUtils as OCR
from src.button_center import ButtonCenter as ButtonCenter
from src.filectl import FileCtl as FileCtl
from src.shortcut import ShortCut as ShortCut
from src.mouse_key import MouseKey as MouseKey
from src.video_utils import VideoUtils as VideoUtils
from src.read_csv import ReadCsv as ReadCsv
from src.pinyin import pinyin as pinyin
from src.sleepx import sleep as sleep
from src.custom_exception import *


class Src(
    CmdCtl,
    ImageUtils,
    FileCtl,
    ShortCut,
    Calculate,
    OCR,
):
    """src"""

    def __init__(
        self,
        name=None,
        description=None,
        config_path=None,
        number=-1,
        check_start=True,
        ui_name=None,
        **kwargs,
    ):
        """dogtail or button center param
        :param kwargs: app_name, desc, number
        """
        self.dog = DogtailUtils(
            name=name,
            description=description,
            number=number,
            check_start=check_start,
            **kwargs,
        )
        ui_name = ui_name if ui_name else name
        # pylint: disable=invalid-name
        self.ui = ButtonCenter(app_name=ui_name, config_path=config_path, number=number)



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_

# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.

# SPDX-License-Identifier: GPL-2.0-only
# pylint: disable=C0114
from time import strftime
from os.path import splitext
from pprint import pprint

import pyscreenshot
from PIL import Image
from PIL import ImageChops

from setting.globalconfig import GlobalConfig
from src.mouse_key import MouseKey

right_action = (
    MouseKey.click,
    MouseKey.right_click,
    MouseKey.double_click,
    MouseKey.move_to,
)


def filter_image(action):
    """
    对比动作前后两张图片，提取不同的部分生成一张新的图片，并返回新图片的路径
    :param action: 动作函数的函数对象
    :return: 新图片的路径
    """
    if not hasattr(action, "__call__"):
        print(f"{action} 不是一个函数对象")
        raise ValueError

    if action not in right_action:
        pprint(f"{action.__name__} 不在 {[i.__name__ for i in right_action]}")
        raise ValueError

    before_img_path = splitext(GlobalConfig.SCREEN_CACHE)[0] + "_before.png"
    pyscreenshot.grab().save(before_img_path)
    before_img = Image.open(before_img_path)
    action()
    after_img_path = splitext(GlobalConfig.SCREEN_CACHE)[0] + "_after.png"
    pyscreenshot.grab().save(after_img_path)
    after_img = Image.open(after_img_path)

    filter_img = ImageChops.difference(before_img, after_img)
    if filter_img.getbbox() is None:
        print(f"{before_img_path}、{after_img_path}两张图片相同")
        return None
    res_img = splitext(GlobalConfig.SCREEN_CACHE)[0] + f'_{strftime("%Y%m%d%H%M%S")}.png'
    filter_img.save(res_img)
    return res_img


if __name__ == "__main__":
    filter_image(MouseKey.right_click)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals
from .config import config
import os
import sys
import time
import traceback

"""
Logging facilities
"""

__author__ = """Ed Rousseau <rousseau@redhat.com>,
Zack Cerza <zcerza@redhat.com,
David Malcolm <dmalcolm@redhat.com>
"""


class TimeStamp(object):
    """
    Timestamp class for file logs

    Generates timestamps tempfiles and log entries
    """
    def __init__(self):
        self.now = "0"
        self.timetup = time.localtime()

    def zeroPad(self, intt, width=2):
        """
        Pads an integer 'int' with zeroes, up to width 'width'.

        Returns a string.

        It will not truncate. If you call zeroPad(100, 2), '100' will be returned.
        """
        if intt < 10 ** width:
            return ("0" * (width - len(str(intt)))) + str(intt)
        else:
            return str(intt)

    # file stamper
    def fileStamp(self, filename, addTime=True):
        """
        Generates a filename stamp in the format of filename_YYYYMMDD-hhmmss.
        A format of filename_YYYYMMDD can be used instead by specifying addTime = False.
        """
        self.now = filename.strip() + "_"
        self.timetup = time.localtime()

        # Should produce rel-eng style filestamps
        # format it all pretty by chopping the tuple
        fieldCount = 3
        if addTime:
            fieldCount = fieldCount + 3
        for i in range(fieldCount):
            if i == 3:
                self.now = self.now + '-'
            self.now = self.now + self.zeroPad(self.timetup[i])
        return self.now

    # Log entry stamper
    def entryStamp(self):
        """
        Generates a logfile entry stamp of YYYY.MM.DD HH:MM:SS
        """
        self.timetup = time.localtime()

        # This will return a log entry formatted string in YYYY.MM.DD HH:MM:SS
        for i in range(6):
            # put in the year
            if i == 0:
                self.now = str(self.timetup[i])
            # Format Month and Day
            elif i == 1 or i == 2:
                self.now = self.now + "." + self.zeroPad(self.timetup[i])
            else:
                # make the " " between Day and Hour and put in the hour
                if i == 3:
                    self.now = self.now + " " + self.zeroPad(self.timetup[i])
                # Otherwise Use the ":" divider
                else:
                    self.now = self.now + ":" + self.zeroPad(self.timetup[i])
        return self.now


class Logger(object):
    """
    Writes entries to standard out.
    """
    stamper = TimeStamp()

    def __init__(self, logName, file=False, stdOut=True):
        """
        name: the name of the log
        file: The file object to log to.
        stdOut: Whether to log to standard out.
        """
        self.logName = logName
        self.stdOut = stdOut
        self.filee = file  # Handle to the logfile

        # store script name
        scriptName = config.scriptName

        # if script name is invalid, use the default one
        # most probably non-reachable code
        if not scriptName:  # pragma: no cover
            scriptName = 'log'

        # use script name as default filename
        self.fileName = scriptName

        # return if file is False or empty string
        # rewrite it for sure to False
        if not self.filee:
            self.filee = False
            return

        # check to see if we can write to the logDir
        if os.path.isdir(config.logDir):
            self.findUniqueName()
        else:
            # If path doesn't exist, raise an exception
            raise IOError("Log path %s does not exist or is not a directory" % config.logDir)

    def findUniqueName(self):
        """
        Generate a logfile name and check if it already exists to obtain a unique one
        """
        self.fileName = config.logDir + self.stamper.fileStamp(self.fileName) + '_' + self.logName
        i = 0

        # Iterate until self.fileName contains new unique file path
        while os.path.exists(self.fileName):
            # Append the file path with counter id
            if i == 0:
                self.fileName = self.fileName + "." + str(i)
            else:
                logsplit = self.fileName.split(".")
                logsplit[-1] = str(i)
                self.fileName = ".".join(logsplit)
            i += 1

    def createFile(self):
        """
        Try to create the file and write the header info
        """
        print("Creating logfile at %s ..." % self.fileName)
        self.filee = open(self.fileName, mode='w')
        self.filee.write("##### " + os.path.basename(self.fileName) + '\n')
        self.filee.flush()

    def log(self, message, newline=True, force=False):
        """
        Hook used for logging messages. Might eventually be a virtual
        function, but nice and simple for now.

        If force is True, log to a file irrespective of config.logDebugToFile.
        """

        # Create log file if file descriptor was not provided to constructor
        # and force or config.logDebugToFile is set to True
        if isinstance(self.filee, bool) and (force or config.logDebugToFile):
            self.createFile()

        # If force or config.logDebugToFile is set to True, write message to file
        if force or config.logDebugToFile:
            if newline:
                self.filee.write(message + str('\n'))
            else:
                self.filee.write(message + str(' '))
            self.filee.flush()

        # If both the self.stdOut and config.logDebugToStdOut are True, write to stdout
        if self.stdOut and config.logDebugToStdOut:
            try:
                print(message)
            except TypeError:
                print(message.decode('utf-8', 'replace'))


class ResultsLogger(Logger):
    """
    Writes entries into the Dogtail log
    """
    def __init__(self, stdOut=True):
        Logger.__init__(self, 'results', file=True, stdOut=stdOut)

    # Writes the result of a test case comparison to the log
    def log(self, entry):
        """
        Writes the log entry. Requires a 1 {key: value} pair dict for an argument or else it will
        throw an exception.
        """
        # We require a 1 key: value dict
        # Strip all leading and trailing witespace from entry dict and convert
        # to string for writing

        if len(entry) == 1:
            key = list(entry.keys())
            value = list(entry.values())
            key = key[0]
            value = value[0]
            entry = str(key) + ":      " + str(value)
        else:
            print("Method argument requires a 1 {key: value} dict. Supplied argument not one {key: value}")
            raise ValueError(entry)

        Logger.log(self, self.stamper.entryStamp() + "      " + entry, force=True)

debugLogger = Logger('debug', config.logDebugToFile)


def exceptionHook(exc, value, tb):  # pragma: no cover
    tbStringList = traceback.format_exception(exc, value, tb)
    tbString = ''.join(tbStringList)
    debugLogger.log(tbString)
    # sys.exc_clear()

sys.excepthook = exceptionHook



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_

# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.

# SPDX-License-Identifier: GPL-2.0-only
# pylint: disable=C0114
from xmlrpc.client import ServerProxy

from setting.globalconfig import GlobalConfig

try:
    import pypinyin

    GET_PINYIN_FROM_RPC = False
except ModuleNotFoundError:
    GET_PINYIN_FROM_RPC = True


def pinyin(word) -> str:
    """
     汉字转化为拼音
    :param word: 待转化的汉语字符串
    :return: 拼音字符串
    """
    if GET_PINYIN_FROM_RPC:
        server = ServerProxy(GlobalConfig.OPENCV_SERVER_HOST, allow_none=True)
        return server.pinyin(word)
    # else:
    _s = ""
    for key in pypinyin.pinyin(word, style=pypinyin.NORMAL):
        _s += "".join(key)
    return _s



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_
# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.
# SPDX-License-Identifier: GPL-2.0-only
from funnylog import logger

from setting import conf
from src import Src
from src.cmdctl import CmdCtl
from src.dogtail_utils import DogtailUtils
from src.remotectl._remote_dogtail_ctl import remote_dogtail_ctl as remote_dogtail_ctl
from src.remotectl._remote_other_ctl import remote_other_ctl as remote_other_ctl
from src.shortcut import ShortCut


class Remote(ShortCut, CmdCtl):
    def __init__(self, ip, user, password, transfer_appname=None, restart_service=False):
        self.user = user
        self.ip = ip
        self.password = password
        self.transfer_appname = transfer_appname
        self.restart_service = restart_service
        self.tmp_obj = None

    def __getattribute__(self, item):
        if not item.startswith("__") and not item.endswith("__"):
            for cls_obj in [ShortCut, CmdCtl]:
                if hasattr(cls_obj, item):
                    self.tmp_obj = {"cls_obj": cls_obj, "item_obj": getattr(cls_obj, item)}
                    while True:
                        if hasattr(cls_obj, item):
                            delattr(cls_obj, item)
                        else:
                            break
        return super().__getattribute__(item)

    def __getattr__(self, item):
        def func(*args, **kwargs):
            ar = ""
            if args:
                for arg in args:
                    ar += f"'{arg}', "
            if kwargs:
                for k, v in kwargs.items():
                    ar += f"{k}='{v}', "
            logger.info(
                f"Remote(user='{self.user}', ip='{self.ip}', password='{self.password}').rctl.{item}({ar.rstrip(', ')})"
            )
            value = None
            try:
                value = getattr(self.rctl, item)(*args, **kwargs)
            finally:
                if self.tmp_obj:
                    setattr(self.tmp_obj["cls_obj"], item, self.tmp_obj["item_obj"])
                    self.tmp_obj = None
            return value

        return func

    @property
    def rdog(self) -> DogtailUtils:
        return remote_dogtail_ctl(
            user=self.user,
            ip=self.ip,
            password=self.password,
            restart_service=self.restart_service,
        )

    def click_element_by_attr(self, element, button=1):
        self.rdog.element_click(element, button)

    def get_element_children_txt(self, element):
        return self.rdog.get_element_children_text(element)

    def get_element_center(self, element):
        return self.rdog.element_center(element)

    @property
    def rctl(self) -> Src:
        return remote_other_ctl(
            user=self.user,
            ip=self.ip,
            password=self.password,
            restart_service=self.restart_service,
        )

    @property
    def rctl_plus(self) -> Src:
        return remote_other_ctl(
            user=self.user,
            ip=self.ip,
            password=self.password,
            transfer_appname=self.transfer_appname,
            restart_service=self.restart_service,
        )

    def find_image(self, image_path):
        _image_path = image_path.replace(conf.HOME, "~", 1)
        return self.rctl_plus.find_image(_image_path)


if __name__ == '__main__':
    a = Remote(
        user="uos",
        ip="10.8.7.55",
        password="1",
    ).rcmd.sudo_run_cmd("systemctl restart lightdm.service")
    # ).run_cmd("ls")
    print(a)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

__author__ = """David Malcolm <dmalcolm@redhat.com>"""


class SearchPath(object):
    """
    Class used by the recording framework (and for more verbose script
    logging) for identifying nodes in a persistent way, independent of the
    style of script being written.

    Implemented as a list of (predicate, isRecursive) pairs, giving the
    'best' way to find the Accessible wrapped by a Node, starting at the
    root and applying each search in turn.

    This is somewhat analagous to an absolute path in a filesystem, except
    that some of searches may be recursive, rather than just searching
    direct children.

    FIXME: try to ensure uniqueness
    FIXME: need some heuristics to get 'good' searches, whatever
    that means
    """

    def __init__(self):
        self.lst = []

    def __str__(self):
        result = "{"
        for (predicate, isRecursive) in self.lst:
            result += "/(%s,%s)" % (predicate.describeSearchResult(), isRecursive)
        return result + "}"

    # We need equality to work so that dicts of these work:
    def __eq__(self, other):
        if len(self.lst) != len(other.lst):
            return False
        else:
            for i in range(len(self.lst)):
                if self.lst[i] != other.lst[i]:
                    return False
        return True

    def append(self, predicate, isRecursive):
        assert predicate
        self.lst.append((predicate, isRecursive))

    def __iter__(self):
        return iter(self.lst)

    def length(self):
        return len(self.lst)

    def makeScriptMethodCall(self):
        """
        Used by the recording system.

        Generate the Python source code that will carry out this search.
        """
        result = ""
        for (predicate, isRecursive) in self.lst:
            # print predicate
            # print self.generateVariableName(predicate)
            result += "." + predicate.makeScriptMethodCall(isRecursive)
        return result

    def getRelativePath(self, other):
        """
        Given another SearchPath instance, if the other is 'below' this
        one, return a SearchPath that describes how to reach it relative
        to this one (a copy of the second part of the list).    Otherwise
        return None.
        """
        i = 0
        for i in range(len(self.lst)):
            if self.lst[i] != other.lst[i]:
                break
        if i > 0:
            # Slice from this point to the end:
            result = SearchPath()
            result.lst = other.lst[i + 1:]

            if False:
                print("....................")
                print("from %s" % self)
                print("to %s" % other)
                print("i=%s" % i)
                print("relative path %s" % result)
                print("....................")

            return result
        else:
            return None

    def getPrefix(self, n):
        """
        Get the first n components of this instance as a new instance
        """
        result = SearchPath()
        for i in range(n):
            result.lst.append(self.lst[i])
        return result

    def getPredicate(self, i):
        (predicate, _) = self.lst[i]
        return predicate
